//////////////////////////////////////////////////
//  Module      : xmpixmap.C  Version 0.1
//  LastEdit    : 1.6.94
//  Description : Verwendung von Xlib-Grafikroutinen und
//                Routinen der Pixmap-Library 
//  Author      : T.Helbig
//////////////////////////////////////////////////
#include <iostream.h>
#include <stdlib.h>

#include <X11/xpm.h>

#include <Xm/Xm.h>
#include <Xm/DrawingA.h>

#define APPCLASSNAME "XMpixmap"

// Pixmaps
#include "bg.xpm"
#include "man.xpm"

void CreateGC();
void CreatePixmaps();
void ExposeCB(Widget, XtPointer, XtPointer);

// Pixmaps
Pixmap        pixBackground;
Pixmap        pixImage;
Pixmap        pixMask;

// Attribute
XpmAttributes attributesBackground;
XpmAttributes attributesImage;

// GC's
GC            gcCopy;
GC            gcAnd;
GC            gcOr;

// Shell-Widget
Widget        wToplevel;

main(int argc, char **argv)
{
  XtAppContext app;

  Widget        wDrawingArea;

  wToplevel = XtVaAppInitialize(&app,
    APPCLASSNAME,
    NULL, 0,
    &argc, argv,
    NULL,
    NULL);

  // Graphische Kontexte erzeugen
  CreateGC();

  // Pixmaps erzeugen
  CreatePixmaps();

  // Icon der Applikation setzen
  XtVaSetValues(wToplevel,
    XmNiconPixmap, pixImage,
    XmNiconMask,   pixMask,
    NULL);

  // Drawing Area erzeugen
  wDrawingArea = XtVaCreateManagedWidget("drawingArea",
    xmDrawingAreaWidgetClass, wToplevel,
    XmNwidth,                 attributesBackground.width,
    XmNheight,                attributesBackground.height,
    NULL);

  // Callback zum Anzeigen der Pixmaps
  XtAddCallback(wDrawingArea, XmNexposeCallback, ExposeCB, NULL);

  XtRealizeWidget(wToplevel);
  XtAppMainLoop(app);
}

void CreateGC()
{
  XGCValues     gcv;

  // Zum Kopieren der Maske
  gcv.function = GXand;
  gcv.graphics_exposures = False;
  gcv.foreground = 0;
  gcv.background = ~0;
  gcAnd = XtGetGC(wToplevel,
    GCFunction | GCGraphicsExposures | GCForeground | GCBackground, &gcv);

  // Zum Kopieren des Bildes
  gcv.function = GXor;
  gcv.graphics_exposures  = False;
  gcOr = XtGetGC(wToplevel, GCFunction | GCGraphicsExposures, &gcv);

  // Zum Kopieren des Hintergrunds
  gcv.function = GXcopy;
  gcv.graphics_exposures = False;
  gcCopy = XtGetGC(wToplevel, GCFunction | GCGraphicsExposures, &gcv);
}

void CreatePixmaps()
{
  int           nStatus;

  // Hintergrund Pixmap erzeugen
  attributesBackground.valuemask = XpmSize;

  nStatus = XpmCreatePixmapFromData(XtDisplay(wToplevel),
    RootWindow(XtDisplay(wToplevel), DefaultScreen(XtDisplay(wToplevel))),
    bg,
    &pixBackground,
    &pixMask,
    &attributesBackground);
    
  if (nStatus != XpmSuccess)
  {
    cerr << "Fataler Fehler in <CreatePixmaps>"
         << "Kann Hintergrund nicht erzeugen" << endl;
    exit(1);
  }

  // Maske wird fuer Hintergrund nicht benoetigt
  if (pixMask)
  {
    XFreePixmap(XtDisplay(wToplevel), pixMask);
  }

  // <pixImage> und <pixMask> erzeugen
  attributesImage.valuemask = XpmSize;

  nStatus = XpmCreatePixmapFromData(XtDisplay(wToplevel),
    RootWindow(XtDisplay(wToplevel), DefaultScreen(XtDisplay(wToplevel))),
    man_xpm,
    &pixImage,
    &pixMask,
    &attributesImage);

  if (nStatus != XpmSuccess || !pixMask)
  {
    cerr << "Fataler Fehler in <CreatePixmaps>"
         << "Kann Image oder Maske nicht erzeugen" << endl;
    exit(1);
  }
}

void ExposeCB(Widget w, XtPointer pCLientData, XtPointer pCallData)
{
  // Wenn das Fenster in mehreren Bereichen ueberdeckt wurde, 
  // hat <pCallData->event->xexpose.count> einen Wert groesser 0.
  // Der Callback wird dann <pCallData->event->xexpose.count> mal 
  // aufgerufen. Da der Fensterinhalt immer komplett neu gezeichnet
  // wird, muss nur der letzte Aufruf beruecksichtigt werden.

  if (((XmDrawingAreaCallbackStruct*)pCallData)->event->xexpose.count > 0)
  {
    return;
  }

  // Hintergrund in DrawingArea kopieren
  XCopyArea(XtDisplay(w),
    pixBackground,
    XtWindow(w),
    gcCopy,
    0, 0,
    attributesBackground.width,
    attributesBackground.height,
    0, 0);

  // Maske mit AND in DrawingArea kopieren
  XCopyPlane(XtDisplay(w),
    pixMask,
    XtWindow(w),
    gcAnd,
    0, 0,
    attributesImage.width,
    attributesImage.height,
    (attributesBackground.width  - attributesImage.width) / 2,
    (attributesBackground.height - attributesImage.height) / 2,
    1L);

  // Image mit OR DrawingArea kopieren
  XCopyArea(XtDisplay(w),
    pixImage,
    XtWindow(w),
    gcOr,
    0, 0,
    attributesImage.width,
    attributesImage.height,
    (attributesBackground.width  - attributesImage.width) / 2,
    (attributesBackground.height - attributesImage.height) / 2);
}

    
   

