//////////////////////////////////////////////////
//  Module      : App.C  Version 0.1
//  LastEdit    : 8.6.93
//  Description : Applikation zum Testen der Sprite
//                Routinen 
//  Author      : T.Helbig
//////////////////////////////////////////////////
#include <stdlib.h>

#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>
#include <Xm/Scale.h>

#include "App.h"
#include "Base.h"
#include "PixmapButton.h"
#include "MySpriteAnimArea.h"

#include "SpriteAnimData.h"

#include "Callback.h" // Callback-Klasse C++Elementfunktionen als
                      // Xt-Callbacks

// Pixmaps
#include "startNormal.xpm"
#include "startArm.xpm"
#include "startInsensitive.xpm"
#include "stopNormal.xpm"
#include "stopArm.xpm"
#include "stopInsensitive.xpm"
#include "pulseNormal.xpm"
#include "pulseArm.xpm"
#include "pulseInsensitive.xpm"

//////////////////////////////////////////////////
// C A p p : : C A p p

CApp::CApp(Widget wParent, const char *szName, Colormap colormap)
  : CBase(szName), m_colormap(colormap)
{
  // MainWindow erzeugen
  m_wBase = XtVaCreateWidget("mainWindow",
    xmMainWindowWidgetClass, wParent,
    NULL);

  // MenuBar erzeugen
  CreateMenuBar();
  CreateWorkWindow();
}
//////////////////////////////////////////////////
// C A p p : : ~C A p p

CApp::~CApp()
{
  if (m_pSpriteAnimArea != NULL)
    delete m_pSpriteAnimArea;
  if (m_pPixButtonStart != NULL)
    delete m_pPixButtonStart;
  if (m_pPixButtonStop != NULL)
    delete m_pPixButtonStop;
  if (m_pPixButtonPulse != NULL)
    delete m_pPixButtonPulse;
}
//////////////////////////////////////////////////
// C A p p : : C r e a t e M e n u B a r

void CApp::CreateMenuBar()
{
  // MenuBar erzeugen
  m_wMenuBar = XmCreateMenuBar(m_wBase, "menuBar", NULL, 0);

  // PullDown-Menue
  m_wPullDownFile = XmCreatePulldownMenu(m_wMenuBar, 
    "filePullDown", NULL, 0);
  XtVaSetValues(m_wPullDownFile,
    XmNtearOffModel, XmTEAR_OFF_ENABLED,
    NULL);

  // PullDown-Menue Ueberschrift
  XtVaCreateManagedWidget("file",
    xmCascadeButtonWidgetClass, m_wMenuBar,
    XmNsubMenuId, m_wPullDownFile,
    NULL);

  // Pulldown-Menu Menuepunkte
  m_wPushBExit = XtVaCreateManagedWidget("exit",
    xmPushButtonWidgetClass, m_wPullDownFile,
    NULL);

  // Callback anhaengen
  AddCallback(m_wPushBExit,
    XmNactivateCallback,
    &CApp::ExitCB,
    (XtPointer) NULL,
    (XtPointer) this);

  XtManageChild(m_wMenuBar);

  // MenuBar beim MainWindow eintragen
  XtVaSetValues(m_wBase,
    XmNmenuBar, m_wMenuBar,
    NULL);
}
//////////////////////////////////////////////////
// C A p p : : C r e a t e W o r k W i n d o w 

void CApp::CreateWorkWindow()
{
  // Form-Widget erzeugen
  m_wFormWorkWindow = XtVaCreateWidget("formWork",
    xmFormWidgetClass, m_wBase,
    NULL);

  // Frame-Widget erzeugen
  m_wFrameWorkWindow = XtVaCreateWidget("frameWork",
    xmFrameWidgetClass, m_wFormWorkWindow,
    XmNtopAttachment,   XmATTACH_FORM,
    XmNleftAttachment,  XmATTACH_FORM,
    NULL);

  // Animationsbereich erzeugen
  m_pSpriteAnimArea = new CMySpriteAnimArea(m_wFrameWorkWindow,
    "animationArea",
    title_xpm,
    spriteObjects,
    nNumObjects,
    m_colormap);
  m_pSpriteAnimArea->Show();

  CreateCommandWindow();

  XtManageChild(m_wFrameWorkWindow);
  XtManageChild(m_wFormWorkWindow);

  // Form-Widget als WorkWindow beim MainWindow eintragen
  XtVaSetValues(m_wBase,
    XmNworkWindow, m_wFormWorkWindow,
    NULL);
}

//////////////////////////////////////////////////
// C A p p : : C r e a t e C o m m a n d W i n d o w

void CApp::CreateCommandWindow()
{
  // Form-Widget erzeugen
  m_wFormCmd = XtVaCreateWidget("formCommand",
    xmFormWidgetClass, m_wFormWorkWindow,
    XmNtopAttachment,  XmATTACH_WIDGET,
    XmNtopWidget,      m_wFrameWorkWindow,
    XmNtopOffset,      10,
    XmNleftAttachment, XmATTACH_FORM,
    XmNleftOffset,     10,
    NULL);
  
  // RowColumn-Widget erzeugen
  m_wRowColumnCmd = XtVaCreateWidget("rowColumnCommand",
    xmRowColumnWidgetClass, m_wFormCmd,
    XmNorientation,          XmHORIZONTAL,
    NULL);

  // Start-Button erzeugen
  m_pPixButtonStart = new CPixmapButton(m_wRowColumnCmd, "pixButtonStart", 
    startNormal_xpm, startArm_xpm, startInsensitive_xpm);
  m_pPixButtonStart->Show();

  // Stop-Button erzeugen
  m_pPixButtonStop = new CPixmapButton(m_wRowColumnCmd, "pixButtonStop", 
    stopNormal_xpm, stopArm_xpm, stopInsensitive_xpm);
  m_pPixButtonStop->Show();
  XtSetSensitive(m_pPixButtonStop->AsWidget(), false);

  // Pulse-Button erzeugen
  m_pPixButtonPulse = new CPixmapButton(m_wRowColumnCmd, "pixButtonPulse", 
    pulseNormal_xpm, pulseArm_xpm, pulseInsensitive_xpm);
  m_pPixButtonPulse->Show();

  // Callbacks anhaengen
  AddCallback(m_pPixButtonStart->AsWidget(),
    XmNactivateCallback,
    &CApp::StartCB,
    (XtPointer) NULL,
    (XtPointer) this);

  AddCallback(m_pPixButtonStop->AsWidget(),
    XmNactivateCallback,
    &CApp::StopCB,
    (XtPointer) NULL,
    (XtPointer) this);

  AddCallback(m_pPixButtonPulse->AsWidget(),
    XmNactivateCallback,
    &CApp::PulseCB,
    (XtPointer) NULL,
    (XtPointer) this);

  // Scale-Widget erzeugen
  m_wScaleSpeed = XtVaCreateManagedWidget("scaleSpeed",
    xmScaleWidgetClass,  m_wFormCmd,
    XmNshowValue,        TRUE,
    XmNorientation,      XmHORIZONTAL,
    XmNleftAttachment,   XmATTACH_WIDGET,
    XmNleftWidget,       m_wRowColumnCmd,
    XmNleftOffset,       10,
    XmNvalue,            m_pSpriteAnimArea->Speed(),
    NULL);

  // Callbacks anhaengen
  AddCallback(m_wScaleSpeed,
    XmNvalueChangedCallback,
    &CApp::SpeedCB,
    (XtPointer) NULL,
    (XtPointer) this);
  AddCallback(m_wScaleSpeed,
    XmNdragCallback,
    &CApp::SpeedCB,
    (XtPointer) NULL,
    (XtPointer) this);

  XtManageChild(m_wRowColumnCmd);
  XtManageChild(m_wFormCmd);
} 

//////////////////////////////////////////////////
// Callbacks
//////////////////////////////////////////////////

//////////////////////////////////////////////////
// C A p p : : S t a r t C B 

void CApp::StartCB(Widget w, XtPointer pClientData, XtPointer pCallData)
{
  m_pSpriteAnimArea->Start();

  XtSetSensitive(m_pPixButtonStart->AsWidget(), False);
  XtSetSensitive(m_pPixButtonStop->AsWidget(), True);
  XtSetSensitive(m_pPixButtonPulse->AsWidget(), False);
}
//////////////////////////////////////////////////
// C A p p : : S t o p C B 

void CApp::StopCB(Widget w, XtPointer pClientData, XtPointer pCallData)
{
  m_pSpriteAnimArea->Stop();

  XtSetSensitive(m_pPixButtonStart->AsWidget(), True);
  XtSetSensitive(m_pPixButtonStop->AsWidget(), False);
  XtSetSensitive(m_pPixButtonPulse->AsWidget(), True);
}
//////////////////////////////////////////////////
// C A p p : : P u l s e C B 

void CApp::PulseCB(Widget w, XtPointer pClientData, XtPointer pCallData)
{
  m_pSpriteAnimArea->Pulse();
}
//////////////////////////////////////////////////
// C A p p : : S p e e d C B 

void CApp::SpeedCB(Widget w, XtPointer pClientData, XtPointer pCallData)
{
  XmScaleCallbackStruct *pCbs = (XmScaleCallbackStruct*) pCallData;

  if (pCbs->value > 0)
  {
    m_pSpriteAnimArea->Speed(pCbs->value);
  }
}
//////////////////////////////////////////////////
// C A p p : : E x i t C B

void CApp::ExitCB(Widget w, XtPointer pClientData, XtPointer pCallData)
{
  exit(0);
}
