//////////////////////////////////////////////////
//  Module      : Sprite.C  Version 0.1
//  LastEdit    : 18.5.94
//  Description : Sprites 
//  Author      : T.Helbig
//////////////////////////////////////////////////
#include <stdlib.h>
#include <string.h>

#include <Xm/Xm.h>
#include "Sprite.h"

//////////////////////////////////////////////////
// statische Klassenkomponenten initialisieren

uint CSprite::m_nMaxId = 0;

GC CSprite::m_gcand  = (GC) 0;
GC CSprite::m_gcor   = (GC) 0;
GC CSprite::m_gccopy = (GC) 0;

//////////////////////////////////////////////////
// C S p r i t e : : C S p r i t e

CSprite::CSprite(Widget widget, Colormap colormap)
 : m_nSpriteId(++m_nMaxId)
{
  assert(widget != (Widget) 0);

  m_pSpriteData = new SpriteData;

  m_widget = widget;
  if (colormap == (Colormap) 0)
  {
    m_colormap = DefaultColormap(XtDisplay(widget), 
      DefaultScreen(XtDisplay(widget)));
  }
  else
  {
    m_colormap = colormap;
  }
  m_pSpriteData->m_pDisplay = XtDisplay(widget);
  m_pSpriteData->m_colormap = m_colormap;

  // Graphische Kontexte erzeugen
  if (m_gcand == (GC) 0)
  {
    XGCValues gcv;

    // Zum Kopieren der Maske
    gcv.function = GXand;
    gcv.graphics_exposures = False;
    gcv.foreground = 0;
    gcv.background = ~0;
    m_gcand = XtGetGC(widget,
      GCFunction | GCGraphicsExposures | GCForeground | GCBackground, &gcv);

    // Zum Kopieren des Bildes
    gcv.function = GXor;
    gcv.graphics_exposures  = False;
    m_gcor = XtGetGC(widget, GCFunction | GCGraphicsExposures, &gcv);

    // Zum Kopieren des Bildes ohne Maske
    gcv.function = GXcopy;
    gcv.graphics_exposures  = False;
    m_gccopy = XtGetGC(widget, GCFunction | GCGraphicsExposures, &gcv);
  }
}
//////////////////////////////////////////////////
// C S p r i t e : : ~ C S p r i t e

CSprite::~CSprite()
{
  // wird <m_pSpriteData> noch von einem anderen Objekt benutzt ?
  if (--m_pSpriteData->m_nReferenceCnt <= 0)
  {
    delete m_pSpriteData;
  }
}
//////////////////////////////////////////////////
// C S p r i t e : : C S p r i t e

CSprite::CSprite(const CSprite& sprite) : m_nSpriteId(++m_nMaxId)
{
  sprite.m_pSpriteData->m_nReferenceCnt++;

  // wird <m_pSpriteData> noch von einem anderen Objekt benutzt ?
  if (--m_pSpriteData->m_nReferenceCnt <= 0)
  {
    delete m_pSpriteData;
  }

  m_pSpriteData = sprite.m_pSpriteData;

  m_widget = sprite.m_widget;
  m_pSpriteData->m_pDisplay = XtDisplay(sprite.m_widget);
  m_colormap = sprite.m_colormap;
  m_pSpriteData->m_colormap = sprite.m_colormap;
}
//////////////////////////////////////////////////
// C S p r i t e : : o p e r a t o r =

const CSprite& CSprite::operator= (const CSprite& sprite)
{
  // Zuweisung an sich selbst, macht nichts
  if (this == &sprite)
  {
    return *this;
  }

  sprite.m_pSpriteData->m_nReferenceCnt++;

  // wird <m_pSpriteData> noch von einem anderen Objekt benutzt ?
  if (--m_pSpriteData->m_nReferenceCnt <= 0)
  {
    delete m_pSpriteData;
  }

  m_pSpriteData = sprite.m_pSpriteData;

  m_widget = sprite.m_widget;
  m_pSpriteData->m_pDisplay = XtDisplay(sprite.m_widget);
  m_colormap = sprite.m_colormap;
  m_pSpriteData->m_colormap = sprite.m_colormap;

  return *this;
}
//////////////////////////////////////////////////
// C S p r i t e : : C r e a t e

bool CSprite::Create(char ***chData, uint nNumSequences, 
  CPoint pos, int nAnimate)
{
  int n;
  int nStatus;

  // Wenn bereits Sprite vorhanden, dann freigeben
  if (m_pSpriteData->m_pixmaps != NULL)
  {
    Detach();
  }

  // Speicherplatz fuer Pixmap-Array allokieren
  m_pSpriteData->m_pixmaps = new Pixmap [nNumSequences];
  m_pSpriteData->m_masks = new Pixmap [nNumSequences];

  // Speicherplatz fuer Attribute Array allokieren
   m_pSpriteData->m_pAttributes = new XpmAttributes [nNumSequences];

  // Sprite Maske und Bild erzeugen 
  for (n = 0; n < nNumSequences; n++)
  {
    // Attribute fuer XpmReadFileToPixmap() setzen
    XtVaGetValues(m_widget,
      XmNdepth, &m_pSpriteData->m_pAttributes[n].depth,
      NULL);
    m_pSpriteData->m_pAttributes[n].visual = DefaultVisual(XtDisplay(m_widget),
      DefaultScreen(XtDisplay(m_widget)));
    m_pSpriteData->m_pAttributes[n].colormap = m_colormap;
    m_pSpriteData->m_pAttributes[n].closeness = 40000;
    m_pSpriteData->m_pAttributes[n].valuemask = XpmColormap | XpmReturnPixels | XpmSize | XpmCloseness | XpmDepth | XpmVisual;

    nStatus = XpmCreatePixmapFromData(XtDisplay(m_widget),
      RootWindow(XtDisplay(m_widget), DefaultScreen(XtDisplay(m_widget))),   
      chData[n],
      &(m_pSpriteData->m_pixmaps[n]),
      &(m_pSpriteData->m_masks[n]),
      &(m_pSpriteData->m_pAttributes[n]));          

    if (nStatus != XpmSuccess)
    {
      if (nStatus == XpmNoMemory)
      {
        cerr << "Fehler in <CSprite::Create> " 
             << "Kein Speicherplatz mehr vorhanden" << endl;
      }
      else if (nStatus == XpmColorFailed)
      {
        cerr << "Fehler in <CSprite::Create> "  
             << "Kann Farben nicht allokieren" << endl << endl;
        cerr << "Versuche <xmsprite -c> !!!" << endl << endl;
      }
      else
      {
        cerr << "Fehler in <CSprite::Create>" 
             << " Kann Sprite nicht erzeugen" << endl;
      }

      return false;
    }
  }
      
  // Anzahl Sequenzen
  m_pSpriteData->m_nNumSequences = nNumSequences;

  // Aktuelle Position
  m_pSpriteData->m_pointCurPos = pos; 

  // Sprite Status
  m_pSpriteData->m_bUpdate = true;
  m_pSpriteData->m_bActive = true;

  // Animation
  m_pSpriteData->m_nAnimate = nAnimate;

  // Sprite Groesse
  m_pSpriteData->m_size.cx = m_pSpriteData->m_pAttributes[0].width;
  m_pSpriteData->m_size.cy = m_pSpriteData->m_pAttributes[0].height;

  return true;
}
//////////////////////////////////////////////////
// C S p r i t e : : S h o w

void CSprite::Show(Drawable d, CPoint pos) const
{
  assert(m_pSpriteData->m_pixmaps != NULL);

  // Maske in Drawable kopieren
  if (m_pSpriteData->m_masks[m_pSpriteData->m_nCurSequence] != (Pixmap)0)
  {
    XCopyPlane(XtDisplay(m_widget), 
      m_pSpriteData->m_masks[m_pSpriteData->m_nCurSequence], d,
      m_gcand, 
      0, 0, 
      m_pSpriteData->m_size.cx, m_pSpriteData->m_size.cy, 
      pos.x, pos.y, 1L);
  }

  // Image in Drawable kopieren
  XCopyArea(XtDisplay(m_widget), 
    m_pSpriteData->m_pixmaps[m_pSpriteData->m_nCurSequence], d,
    ((m_pSpriteData->m_masks[m_pSpriteData->m_nCurSequence] != (Pixmap)0) ? 
      m_gcor : m_gccopy), 
    0, 0, 
    m_pSpriteData->m_size.cx, m_pSpriteData->m_size.cy, 
    pos.x, pos.y);
}
//////////////////////////////////////////////////
// C S p r i t e : : D e t a c h 

void CSprite::Detach()
{
  if (--m_pSpriteData->m_nReferenceCnt <= 0)
    delete m_pSpriteData;

  m_pSpriteData = new SpriteData;

  m_pSpriteData->m_pDisplay = XtDisplay(m_widget);
  m_pSpriteData->m_colormap = m_colormap;
}

