//////////////////////////////////////////////////
//  Module      : SpriteAnim.h  Version 0.1
//  LastEdit    : 16.5.93
//  Description : Animation von Sprites
//  Author      : T.Helbig
//////////////////////////////////////////////////
#include <stdlib.h>        // abs()
#include <assert.h>

#include "Callback.h"      // Callback-Klasse 
#include "SpriteAnim.h"


//////////////////////////////////////////////////
// statische Klassenkomponenten initialisieren

GC CSpriteAnim::m_gc  = (GC) 0;

//////////////////////////////////////////////////
// C S p r i t e A n i m : : C S p r i t e A n i m

CSpriteAnim::CSpriteAnim(Widget w, CSize size, Pixmap background)
{
  int nDepth;

  assert(XmIsDrawingArea(w));
  assert(background != (Pixmap) 0);

  m_widget = w;
  m_size = size; 
  m_pixBackground = background;
  m_sizeOffscreen = size;
  XtVaGetValues(m_widget,
    XmNdepth, &nDepth,
    NULL);
  m_pixOffscreen = XCreatePixmap(XtDisplay(m_widget), 
    RootWindow(XtDisplay(m_widget), DefaultScreen(XtDisplay(m_widget))),
    size.cx, size.cy,
    nDepth);
  m_refresh = false;

  // Callback fuer Redraw anhaengen
  AddCallback(m_widget,
    XmNexposeCallback,
    &CSpriteAnim::ExposureCB,
    (XtPointer) NULL,
    (XtPointer) this);

  // Wenn noch kein GC vorhanden, dann Erzeugen
  if (m_gc == (GC) 0)
  {
    XGCValues gcv;
    gcv.function = GXcopy;
    gcv.graphics_exposures = false;
    m_gc = XtGetGC(m_widget, GCFunction | GCGraphicsExposures, 
      &gcv);
  }
}
//////////////////////////////////////////////////
// C S p r i t e A n i m : : A d d

void CSpriteAnim::Add(const CSprite& sprite)
{
  m_sprites.Add(&sprite);
}
//////////////////////////////////////////////////
// C S p r i t e A n i m : : A n i m a t e

void CSpriteAnim::Animate()
{
  int     n;
  int     i;
  int     nNumSprites;
  CRect   rectUpdate1;
  CRect   rectUpdate2;
  CRect   rectIntersect;
  CRect   rectUnion;
  CPoint  pointNewPosInOffscreen;

  // Zaehlt die Anzahl der Aufrufe von Animate
  m_intcounter++;

  // Kompletten Bereich neu Zeichnen ?
  if (m_refresh)
  {
    Exposure();
  }

  nNumSprites = m_sprites.NumElems();
  // Ermittle Sprites, die Update benoetigen
  for (n = 0; n < nNumSprites; n++)
  {
    CSprite& sprite = (CSprite&) *m_sprites[n];
    if (abs(sprite.Animate()))
    {
      sprite.Update(true);
    }
  }

  for (n = 0; n < nNumSprites; n++)
  {
    CSprite& sprite1 = (CSprite&) *m_sprites[n]; 
    if (sprite1.Update())
    {
      ///////////////////////////////////////////
      // 1. Sprite braucht Update aufgrund von Bewegung 
      //    und/oder wird animiert.
      //    Markiere dieses und alle ueberlappenden
      //    Sprites mit <sprite.Overlap(true)>

      sprite1.Overlap(true);
      rectUpdate1 = sprite1.MoveExtent(); 
      for (i = 0; i < nNumSprites; i++)
      {
        // Suche Sprites, die innerhalb von <rectUpdate1> liegen
        CSprite& sprite2 = (CSprite&) *m_sprites[i];
        if (!sprite2.Overlap() && sprite2.Update())
        {
          rectUpdate2 = sprite2.MoveExtent();
          if (rectUpdate2.Overlap(rectUpdate1))
          {
            // Sprites ueberlappen sich
            rectUnion.UnionRect(rectUpdate1, rectUpdate2);
            if (!(rectUnion == rectUpdate1))
            {
              // Union beider Rechtecke ist groesser als bisheriges Rechteck
              rectUpdate1 = rectUnion;
              i = 0;
            }
            sprite2.Overlap(true);
          }     
        }
      }

      // Teste, das <rectUpdate1> nicht den Bereich der DrawingArea
      // verlaesst. (0, 0, m_size.cx, m_size.cy).
      if (rectUpdate1.Overlap(CRect(0, 0, 0 + m_size.cx, 0 + m_size.cy)))
      {
        rectUpdate1.left = Max(0, rectUpdate1.left);
        rectUpdate1.top = Max(0, rectUpdate1.top);
        rectUpdate1.right = Min(0 + m_size.cx, rectUpdate1.right);
        rectUpdate1.bottom = Min(0 + m_size.cy, rectUpdate1.bottom);
      }
      else
        // Der abzudatende Bereich und die Drawing Area ueberlappen sich
        // nicht, also braucht nichts gemacht zu werden. Kann zwar eigentlich
        // nicht passieren. Wenn doch schadet ein continue aber auch nicht.
        continue;

      ////////////////////////////////////////////
      // 2. Suche nach statischen Sprites, die von <rectUpdate1>
      //    ueberdeckt werden und markiere diese ebenfalls mit
      //    <sprite.Overlap(true)>

      for (i = 0; i < nNumSprites; i++)
      {
        CSprite& sprite2 = (CSprite&) *m_sprites[i];
        if (!sprite2.Overlap() && !sprite2.Update() && sprite2.Active())
        {
          rectUpdate2 = sprite2.Extent();
          if (rectUpdate2.Overlap(rectUpdate1))
          {
            sprite2.Overlap(true);
          }
        }
      }

      ////////////////////////////////////////////
      // 3. Kopiere aus dem Hintergrund das Rechteck
      //    <rectUpdate1> in die Offscreen-Pixmap

      XCopyArea(XtDisplay(m_widget),
        m_pixBackground, m_pixOffscreen,
        m_gc,
        rectUpdate1.left, rectUpdate1.top,
        (unsigned int)rectUpdate1.Width(), (unsigned int)rectUpdate1.Height(),
        0, 0);

      ////////////////////////////////////////////
      // 4. Kopiere alle mit <sprite.Overlap(true)> markierten Sprites
      //    an ihre neue Position in die Offscreen-Pixmap

      for (i = 0; i < nNumSprites; i++)
      {
        CSprite& sprite2 = (CSprite&) *m_sprites[i];
        if (sprite2.Overlap())
        {
          // Soll Sprite animiert werden ?
          if (abs(sprite2.Animate()) && 
              !(m_intcounter % abs(sprite2.Animate())))
          {
            if (sprite2.Animate() > 0)
            {
              // Forwaertsanimation  
              sprite2.Sequence(sprite2.Sequence() + 1);
            }
            else
            {
              // Rueckwaertsanimation
              sprite2.Sequence(sprite2.Sequence() - 1);
            }
          }

          // Sprite Position updaten 
          if (sprite2.IsMoving())
          {
            sprite2.Position(sprite2.Position() + 
              CPoint(sprite2.DeltaLastPos()));
            sprite2.ResetMoves();
          }

          // Berechne Position des Sprite im Backing Store
          pointNewPosInOffscreen = 
            sprite2.Position() - CPoint(rectUpdate1.left, rectUpdate1.top);

          // Uebertrage Sprite in Backing Store
          sprite2.Show(m_pixOffscreen, pointNewPosInOffscreen);

          // Sprite Zustand updaten
          sprite2.Update(false);
          sprite2.Overlap(false);
        }
      }

      ////////////////////////////////////////////
      // 5. Kopiere die Offscreen-Pixmap in die DrawingArea

      XCopyArea(XtDisplay(m_widget),
        m_pixOffscreen, XtWindow(m_widget),
        m_gc,
        0, 0,
        (unsigned int)rectUpdate1.Width(), (unsigned int)rectUpdate1.Height(),
        rectUpdate1.left, rectUpdate1.top);
    }
  }
}
//////////////////////////////////////////////////
// C S p r i t e A n i m : : E x p o s u r e C B 

void CSpriteAnim::ExposureCB(Widget w, XtPointer pClientData, 
  XtPointer pCallData)
{
  if (((XmDrawingAreaCallbackStruct*)pCallData)->event->xexpose.count > 0)
  {
    return;
  }
  Exposure();
}
//////////////////////////////////////////////////
// C S p r i t e A n i m : : E x p o s u r e 

void CSpriteAnim::Exposure()
{
  int     n;
  CPoint  spritePos;

  // <m_pixBackground> nach <m_pixOffscreen> kopieren
  XCopyArea(XtDisplay(m_widget), 
    m_pixBackground, m_pixOffscreen,
    m_gc,
    0, 0,
    m_size.cx, m_size.cy,
    0, 0);

  // Zeichne die aktiven Sprites in <m_pixOffscreen>
  for (n = 0; n < m_sprites.NumElems(); n++)
  {
    if (m_sprites[n]->Active())
    {
       m_sprites[n]->Show(m_pixOffscreen, m_sprites[n]->Position()); 
    }
    
    m_sprites[n]->Overlap(false);
  }

  // Kopiere <m_pixOffscreen> in die Drawing Area.
  XCopyArea(XtDisplay(m_widget),
    m_pixOffscreen, XtWindow(m_widget),
    m_gc,
    0, 0,
    m_sizeOffscreen.cx, m_sizeOffscreen.cy,
    0, 0);

  m_refresh = false;
}

