//////////////////////////////////////////////////
//  Module      : SpriteAnimArea.h  Version 0.1
//  LastEdit    : 23.5.93 
//  Description : Drawing Area zur Sprite Animation 
//  Author      : T.Helbig
//////////////////////////////////////////////////
#ifndef _SPRITE_ANIM_AREA_H
#define _SPRITE_ANIM_AREA_H

#include <assert.h>

#include "Types.h"
#include "SpriteAnim.h"
#include "Sprite.h"
#include "SpriteTimer.h"
#include "Point.h"
#include "Size.h"
#include "Base.h"

// Struktur fuer den Aufruf von CSpriteAnimArea::CSpriteAnimArea
struct SpriteStructData
{
  char ***m_pszSequenceNames;
  int     m_nNumSequences;
  CPoint  m_pointStartPos;
  int     m_nAnimation;

  SpriteStructData(char ***pszSequenceNames,  
    int nNumSequences,
    CPoint pointStartPos, 
    int nAnimation)
  {
    assert (pszSequenceNames != NULL);
    m_pszSequenceNames = pszSequenceNames; 
    m_nNumSequences = nNumSequences;
    m_pointStartPos = pointStartPos; 
    m_nAnimation = nAnimation; 
  }
};

class CSpriteTimer;

//////////////////////////////////////////////////
// CSpriteAnimArea - Klasse zum Erzeugen von Sprite-
//                   Animationen

class CSpriteAnimArea : public CBase
{
  friend CSpriteTimer;

  protected:
    CSpriteAnim  *m_pSpriteAnim;     // Animations Objekt 

    CSprite     **m_pSprites;        // Sprite Objekte
    int           m_nNumSprites;     // Anzahl der Sprites

    Pixmap        m_pixmapBackground;// Hintergrund Bild
    XpmAttributes m_xpmAttributes;   // Attribute zu <m_pixmapBackground>

    Colormap      m_colormap;        // Colormap

    CSpriteTimer *m_pTimer;          // Timer Objekt

    CPoint        m_pointMousePos;   // MousePosition

    bool          m_bGood;           // True = alles OK, False = Fehler

  protected:
    // MousePosition mit EventHandler ermitteln
    void MousePosEH(Widget, XtPointer, XEvent *, Boolean *);
 
    // Funktion zur Bewegung der Sprites, muss von abgeleiteter 
    // Klasse implementiert werden.
    virtual void MoveSprites() = 0;         // Bewegung der Sprites 

  public:
    CSpriteAnimArea(Widget                  wParent, 
                    const char             *szName, 
                    const char            **pBackgroundData, 
                    const SpriteStructData *spriteObjects, 
                    uint                    nNumObjects, 
                    Colormap                colormap = (Colormap)0);
    virtual ~CSpriteAnimArea(); 

    // Status abfragen
    bool Good() const;

    // Hoehe/Breite des Animationsbereichs
    int Width() const; 
    int Height() const;

    // Position der Maus im Animationsbereich
    const CPoint& MousePos() const;

    // Steuerung der Animation
    void Start() const;                // Animation starten 
    void Stop() const;                 // Animation stoppen 
    void Pulse() const;                // Einzelschritt
    uint Speed() const;                // Geschwindigkeit erfragen
    void Speed(uint nSpeed);           // Geschwindigkeit setzen 
};
//////////////////////////////////////////////////
// CSpriteAnimArea:: G o o d

inline bool CSpriteAnimArea::Good() const
{
  return m_bGood;
}
//////////////////////////////////////////////////
// CSpriteAnimArea:: W i d t h

inline int CSpriteAnimArea::Width() const
{
  return (int) m_xpmAttributes.width;
}
//////////////////////////////////////////////////
// CSpriteAnimArea:: H e i g h t 

inline int CSpriteAnimArea::Height() const
{
  return (int) m_xpmAttributes.height;
}
//////////////////////////////////////////////////
// CSpriteAnimArea:: M o u s e P o s 

inline const CPoint& CSpriteAnimArea::MousePos() const
{
  return m_pointMousePos;
}
//////////////////////////////////////////////////
// CSpriteAnimArea:: S t a r t

inline void CSpriteAnimArea::Start() const
{
  assert(m_pTimer != NULL);
  
  m_pTimer->Start();
}
//////////////////////////////////////////////////
// CSpriteAnimArea:: S t o p  

inline void CSpriteAnimArea::Stop() const
{
  assert(m_pTimer != NULL);
  
  m_pTimer->Stop();
}
//////////////////////////////////////////////////
// CSpriteAnimArea:: P u l s e 

inline void CSpriteAnimArea::Pulse() const
{
  assert(m_pTimer != NULL);
  
  m_pTimer->Pulse();
}
//////////////////////////////////////////////////
// CSpriteAnimArea:: S p e e d 

inline uint CSpriteAnimArea::Speed() const
{
  assert(m_pTimer != NULL);
  
  return m_pTimer->Speed();
}
//////////////////////////////////////////////////
// CSpriteAnimArea:: S p e e d 

inline void CSpriteAnimArea::Speed(uint nSpeed)
{
  assert(m_pTimer != NULL);
  
  m_pTimer->Speed(nSpeed);
}

#endif // _SPRITE_ANIM_AREA
