//////////////////////////////////////////////////
//  Module      : Timer.C  Version 0.1
//  LastEdit    : 21.05.94 
//  Description : Timer 
//  Author      : T.Helbig
//////////////////////////////////////////////////
#include <assert.h>

#include "Timer.h"

//////////////////////////////////////////////////
// CTimer - Klasse zur Steuerung von Ablauefen mit
//          einem Timer 

//////////////////////////////////////////////////
// C T i m e r : : C T i m e r

CTimer::CTimer(Widget w, uint nSpeed)
{
  assert(nSpeed > 0);   
  assert(w != NULL);

  // Das Intervall fuer den Aufruf der Timer Funktion muss in [ms]
  // angegeben werden. 
  m_dwDeltaTime = 1000 / nSpeed; 
  m_nSpeed = nSpeed;
  m_intervallId = 0L;
  m_widget = w;
}
//////////////////////////////////////////////////
// C T i m e r : : ~ C T i m e r 

CTimer::~CTimer()
{
  if (m_intervallId)
  {
    XtRemoveTimeOut(m_intervallId);
  }
}
//////////////////////////////////////////////////
// C T i m e r : : S t o p

void CTimer::Stop()
{
  if (m_intervallId) 
  {
    XtRemoveTimeOut(m_intervallId);
  }
  m_intervallId = 0L;
}
//////////////////////////////////////////////////
// C T i m e r : : S t a r t

void CTimer::Start()
{
  m_intervallId = XtAppAddTimeOut(XtWidgetToApplicationContext(m_widget),
    m_dwDeltaTime,
    &CTimer::TimerCB,
    (XtPointer) this);
}
//////////////////////////////////////////////////
// C T i m e r : : P u l s e

void CTimer::Pulse()
{
  TimerFunction();
}
//////////////////////////////////////////////////
// C T i m e r : : S p e e d

void CTimer::Speed(uint nSpeed)
{
  assert(nSpeed > 0);

  m_dwDeltaTime = 1000 / nSpeed;
  m_nSpeed = nSpeed;

  if (m_intervallId)
  {
    XtRemoveTimeOut(m_intervallId);
    m_intervallId = XtAppAddTimeOut(XtWidgetToApplicationContext(m_widget),
      m_dwDeltaTime,
      &CTimer::TimerCB,
      (XtPointer) this);
  }
}
//////////////////////////////////////////////////
// C T i m e r : : S p e e d

uint CTimer::Speed()
{
  return m_nSpeed;
} 
//////////////////////////////////////////////////
// C T i m e r : : T i m e r C B

void CTimer::TimerCB(XtPointer pClientData, XtIntervalId *id)
{
  CTimer* pObj = (CTimer*) pClientData;
  pObj->Timer();
}
//////////////////////////////////////////////////
// C T i m e r : : T i m e r 

void CTimer::Timer()
{
  TimerFunction();

  m_intervallId = XtAppAddTimeOut(XtWidgetToApplicationContext(m_widget),
    m_dwDeltaTime,
    &CTimer::TimerCB,
    (XtPointer) this);
}

  
