//////////////////////////////////////////////////
//  Module      : Callback.h  Version 0.1
//  LastEdit    : 5.6.94
//  Description : Klasse CCallback
//
//                - Benutzung von C++ Elementfunktionen
//                  in Verbindung mit Xt Callbacks 
//
//  Author      : T.Helbig
//////////////////////////////////////////////////
#ifndef _Callback_h
#define _Callback_h

#include <Xm/Xm.h>

typedef void (*CallbackProc)(void *, Widget, XtPointer, XtPointer);

//////////////////////////////////////////////////
// CCallback - C++ Elementfunktionen und Xt Callbacks

class CCallback
{
  protected:
    // Widget bei dem Callback registriert ist
    Widget m_widget;

    // Callback Type
    char *m_szCallbackName;

    // Client-Data
    XtPointer m_pClientData;

    // Pointer auf Elmentfunktion 
    void *m_pCallbackProc;

    // Pointer auf das Objekt, zu der die Elementfunktion gehoert 
    void *m_pObject;

  private:
    // Callback Verteiler
    static void CallbackHandler(Widget w, XtPointer pClientData,
      XtPointer pCallData)
    {
      CCallback *pObject = (CCallback *) pClientData;
      pObject->Execute(w, pObject->m_pClientData, pCallData);
    }

    // Aufruf der Elementfunktion
    void Execute(Widget w, XtPointer pClientData, XtPointer pCallData)
    {
      CallbackProc pFunction = (CallbackProc) m_pCallbackProc;
      (pFunction)(m_pObject, w, pClientData, pCallData);
    }

  public:
    // Konstruktor
    CCallback(Widget     w, 
              char      *szCallbackName, 
              void      *pCallbackProc, 
              XtPointer  pClientData, 
              void      *pObject)
      : m_widget(w), 
        m_szCallbackName(szCallbackName), 
        m_pCallbackProc(pCallbackProc),
        m_pClientData(pClientData), 
        m_pObject(pObject) 
    {
      XtAddCallback(m_widget,
        m_szCallbackName,
        &CCallback::CallbackHandler,
        (XtPointer) this);
      // Destroy Handler fuer den Fall, dass Widget mit 
      // XtDestroy zerstoert wird ??
    }

    // Destruktor
    ~CCallback()
    {
      XtRemoveCallback(m_widget, 
        m_szCallbackName,
        &CCallback::CallbackHandler,
        (XtPointer) this);
    }

  public:
};
//////////////////////////////////////////////////
// A d d C a l l b a c k 
// Callback mit Klasse CCallback registrieren 

inline CCallback *AddCallback(Widget     w, 
                       char      *szCallbackName,
                       void      *pCallbackProc, 
                       XtPointer  pClientData,
                       void      *pObject)
{
  return new CCallback(w, 
    szCallbackName, 
    pCallbackProc, 
    pClientData,
    pObject);
}

#endif // _Callback_h
