//////////////////////////////////////////////////
//  Module      : Event.h  Version 0.1
//  LastEdit    : 5.6.94
//  Description : Klasse CEvent
//
//                - Benutzung von C++ Elementfunktionen
//                  als EventHandler
//
//  Author      : T.Helbig
//////////////////////////////////////////////////
#ifndef _Event_h
#define _Event_h

#include <Xm/Xm.h>

typedef void (*EventProc)(void *, Widget, XtPointer, XEvent *, Boolean *);

//////////////////////////////////////////////////
// CEvent - C++ Elementfunktionen und Xt EventHandler 

class CEvent
{
  protected:
    // Widget bei dem EventHandler registriert ist
    Widget m_widget;

    // Callback Type
    EventMask m_eventMask;

    // Eventhandler fuer Maskable/Nonmaskable Events
    Boolean m_bNonMaskable;

    // Client-Data
    XtPointer m_pClientData;

    // Pointer auf Elmentfunktion 
    void *m_pEventProc;

    // Pointer auf das Objekt, zu der die Elementfunktion gehoert 
    void *m_pObject;

  private:
    // Event Verteiler
    static void EventHandler(Widget w, XtPointer pClientData,
      XEvent *pEvent, Boolean *pFlag)
    {
      CEvent *pObject = (CEvent *) pClientData;
      pObject->Execute(w, pObject->m_pClientData, pEvent, pFlag);
    }

    // Aufruf der Elementfunktion
    void Execute(Widget w, XtPointer pClientData, XEvent *pEvent,
      Boolean *pFlag)
    {
      EventProc pFunction = (EventProc) m_pEventProc;
      (pFunction)(m_pObject, w, pClientData, pEvent, pFlag);
    }

  public:
    // Konstruktor
    CEvent(Widget     w, 
           EventMask  eventMask, 
           Boolean    bNonMaskable,
           void      *pEventProc, 
           XtPointer  pClientData, 
           void      *pObject)
      : m_widget(w), 
        m_eventMask(eventMask), 
        m_bNonMaskable(bNonMaskable),
        m_pEventProc(pEventProc),
        m_pClientData(pClientData), 
        m_pObject(pObject) 
    {
      XtAddEventHandler(m_widget,
        m_eventMask,
        m_bNonMaskable,
        &CEvent::EventHandler,
        (XtPointer) this);
      // Destroy Handler fuer den Fall, dass Widget mit 
      // XtDestroy zerstoert wird ??
    }

    // Destruktor
    ~CEvent()
    {
      XtRemoveEventHandler(m_widget, 
        m_eventMask,
        m_bNonMaskable,
        &CEvent::EventHandler,
        (XtPointer) this);
    }
};
//////////////////////////////////////////////////
// A d d E v e n t H a n d l e r 
// Callback mit Klasse CCallback registrieren 

inline CEvent *AddEventHandler(Widget     w, 
                               EventMask  eventMask,
                               Boolean    bNonMaskable,
                               void      *pEventProc, 
                               XtPointer  pClientData,
                               void      *pObject)
{
  return new CEvent(w, 
    eventMask, 
    bNonMaskable, 
    pEventProc,
    pClientData,
    pObject);
}

#endif // _Event_h
