//////////////////////////////////////////////////
//  Module      : PixmapButton.C  Version 0.1
//  LastEdit    : 8.6.94
//  Description : PushButton mit farbiger Pixmap 
//  Author      : T.Helbig
//////////////////////////////////////////////////
#include <assert.h>

#include <X11/xpm.h>

#include <Xm/Xm.h>
#include <Xm/PushB.h>

#include "PixmapButton.h"

//////////////////////////////////////////////////
// CPixmapButton:: C P i x m a p B u t t o n

CPixmapButton::CPixmapButton(Widget wParent, const char *szName, 
  const char **chData, const char **chDataArm, 
  const char **chDataInsensitive) 
  : CBase(szName)
{
  Pixmap pixImage;
  Pixmap pixArm;
  Pixmap pixInsensitive;

  XmString xmstring;

  assert(szName != NULL);
  assert(chData != NULL);

  // Erzeuge PushButton-Widget
  m_wBase = XtVaCreateManagedWidget(szName,
    xmPushButtonWidgetClass, wParent,
    NULL);

  if (CreatePixmap(pixImage, chData, false))
  {
    XtVaSetValues(m_wBase,
      XmNlabelType, XmPIXMAP,
      XmNlabelPixmap, pixImage,
      NULL);

    if (CreatePixmap(pixArm, chDataArm, true))
    {
      XtVaSetValues(m_wBase,
        XmNarmPixmap, pixArm,
        NULL);
    }
    if (CreatePixmap(pixInsensitive, chDataInsensitive, false))
    {
      XtVaSetValues(m_wBase,
        XmNlabelInsensitivePixmap, pixInsensitive,
        NULL);
    }
  }
  else
  {
    xmstring = XmStringCreateSimple("no Pixmap");
    XtVaSetValues(m_wBase,
      XmNlabelString, xmstring,
      NULL);
    XmStringFree(xmstring);
  }
}
//////////////////////////////////////////////////
// C P i x m a p B u t t o n : : C r e a t e P i x m a p

bool CPixmapButton::CreatePixmap(Pixmap& pixmap, const char **chData, 
  bool flag)
{
  Pixmap         pixMask;
  XpmColorSymbol colorSymbol[1];
  Pixel          bg;
  int            nStatus;
  XpmAttributes  attributes;

  if (chData == NULL)
  {
    return false;
  }

  // Erfrage Hintergrund und Colormap des PushButton-Widgets
  XtVaGetValues(m_wBase, 
    flag ? XmNarmColor : XmNbackground, &bg,
    XmNcolormap, &attributes.colormap,
    NULL);

  // Farbe mit symbolischem Namen "background" in der Xpm-Datei soll
  // durch Hintergrundfarbe des PushButton-Widgets ersetzt werden
  colorSymbol[0].name = "background";
  colorSymbol[0].value = NULL;
  colorSymbol[0].pixel = bg;

  attributes.colorsymbols = colorSymbol;
  attributes.numsymbols = 1;

  // Setze Valuemask von <attributes>
  attributes.valuemask = XpmColorSymbols | XpmColormap;

  // Erzeuge die Pixmap
  nStatus = XpmCreatePixmapFromData(XtDisplay(m_wBase),
    RootWindow(XtDisplay(m_wBase), DefaultScreen(XtDisplay(m_wBase))),
    chData,
    &pixmap,
    &pixMask,
    &attributes);

  // Wenn Maske erzeugt wurde, d.h. in der Xpm-Datei transparente
  // Pixel definiert sind, loesche sie. 
  if (pixMask)
  {
    XFreePixmap(XtDisplay(m_wBase), pixMask);
  }

  // Attribute werden nicht gebraucht
  XpmFreeAttributes(&attributes);

  if (nStatus != XpmSuccess)
  {
    return true;
  }
    
  return true;
}
