//////////////////////////////////////////////////
//  Module      : Array.h  Version 0.1
//  LastEdit    : 18.5.93
//  Description : Template Klasse Array 
//  Author      : T.Helbig
//////////////////////////////////////////////////
#ifndef _ARRAY_H
#define _ARRAY_H

#include <stdlib.h>
#include <string.h>
#include <iostream.h>

#include "Types.h"         // Definition des Datentyps <bool>

//////////////////////////////////////////////////
// CArray: Ein Array ist eine Menge, indem die
//         Reihenfolge der Element nicht definiert
//         ist.
template <class T>
class CArray
{
  private:
    T *m_pElems;
    int m_nSize;
    int m_nNumElems;

  public:
    CArray();
    ~CArray();

    // Aufgrund dynamischer Komponenten
    CArray (const CArray<T>&);
    const CArray<T>& operator= (const CArray<T>&);

    // Fuegt das gegebenen Objekt in das Array ein
    void Add(T&);

    // Anzahl der Elemente im Array 
    int NumElems() const;

    // operator [] fuer Variablen
    T& operator [] (int);

    // operator [] fuer Konstanten 
    const T& operator [] (int) const;
};

//- Klasse CArray<T> -----------------------------

//////////////////////////////////////////////////
// C A r r a y < T > : : C A r r a y
// Konstruktor

template <class T>
inline CArray<T>::CArray()
{
  m_nNumElems = 0;
  m_nSize = 10;
  m_pElems = new T [m_nSize];
}

//////////////////////////////////////////////////
// C A r r a y < T > : : C A r r a y
// Copy-Konstruktor

template <class T>
inline CArray<T>::CArray(const CArray<T>& s)
{
  m_nNumElems = s.m_nNumElems;
  m_nSize = m_nNumElems;
  m_pElems = new T [m_nSize];

  for (int i = 0; i < m_nNumElems; i++)
  {
    m_pElems[i] = s.m_pElems[i];
  }
}

//////////////////////////////////////////////////
// C A r r a y < T > : : ~ C A r r a y ( )
// Destruktor

template <class T>
inline CArray<T>::~CArray()
{
  delete [] m_pElems;
}

//////////////////////////////////////////////////
// C A r r a y < T > : : o p e r a t o r =
// Zuweisungs-Operator

template <class T>
inline const CArray<T>& CArray<T>::operator= (const CArray<T>& s)
{
  if (&s == this)
  {
    return *this;
  }

  m_nNumElems = s.m_nNumElems;

  if (m_nSize < m_nNumElems)
  {
    delete [] m_pElems;
    m_nSize = m_nNumElems;
    m_pElems = new T [m_nSize];
  }

  for (int i = 0; i < m_nNumElems; i++)
  {
    m_pElems[i] = s.m_pElems[i];
  }

  return *this;
}

//////////////////////////////////////////////////
// C A r r a y < T > : : A d d
// Fuegt <x> in das Array ein.

template <class T>
inline void CArray<T>::Add(T& x)
{
  if (m_nSize <= m_nNumElems)
  {
    T *tmp = m_pElems;
    m_nSize *= 2;
    m_pElems = new T [m_nSize];
    for (int i = 0; i < m_nNumElems; i++)
    {
      m_pElems[i] = tmp[i];
    }
    delete [] tmp;
  }
  m_pElems[m_nNumElems++] = x;
}

//////////////////////////////////////////////////
// C A r r a y < T > : : N u m E l e m s 
// Anzahl der Elemente 

template <class T>
inline int CArray<T>::NumElems() const
{
  return m_nNumElems; 
}

//////////////////////////////////////////////////
// C A r r a y < T > : : o p e r a t o r [ ]

template <class T>
inline T& CArray<T>::operator [] (int nIndex)
{
  if (nIndex >= m_nNumElems || nIndex < 0)
  {
    cerr << "unerlaubter Index " << nIndex << " in"
         << " CArray::operator" << endl;
    exit(1);
  }
  return m_pElems[nIndex];
}

//////////////////////////////////////////////////
// C A r r a y < T > : : o p e r a t o r [ ]

template <class T>
inline const T& CArray<T>::operator [] (int nIndex) const
{
  if (nIndex >= m_nNumElems || nIndex < 0)
  {
    cerr << "unerlaubter Index " << nIndex << " in"
         << " CArray::operator" << endl;
    exit(1);
  }
  return m_pElems[nIndex];
}
#endif // _ARRAY_H
