//////////////////////////////////////////////////
//  Module      : Rect.h  Version 0.1
//  LastEdit    : 16.5.93
//  Description : Klasse fuer Rechtecke 
//  Author      : T.Helbig
//////////////////////////////////////////////////
#ifndef _RECT_H
#define _RECT_H

#include <iostream.h>
#include <assert.h>

#include "Types.h"
#include "Point.h"
#include "Misc.h"

typedef struct _RECT
{
  int left;
  int top;
  int right;
  int bottom;
} RECT;

//////////////////////////////////////////////////
// CRect - Klasse fuer Rechtecke

class CRect : public _RECT
{
  public:
    CRect();
    CRect(int nLeft, int nTop, int nRight, int nBottom);
    CRect(const RECT& rect);

    int Width() const;
    int Height() const;
    CPoint& TopLeft() const;
    CPoint& BottomRight() const;

    bool PointInRect(const POINT& point) const;
    bool Overlap(const RECT& rect) const;

    void UnionRect(const RECT& rect1, const RECT& rect2);
    void IntersectRect(const RECT& rect1, const RECT& rect2);

    bool operator== (const RECT& rect) const;
    bool operator!= (const RECT& rect) const;
    void operator+= (POINT point);
    void operator-= (POINT point);  

    CRect operator+ (POINT point) const;
    CRect operator- (POINT point) const;
};

inline CRect::CRect()
{
  left = top = right = bottom = 0;
}

inline CRect::CRect(int nLeft, int nTop, int nRight, int nBottom)
{
  left = nLeft; top = nTop;
  right = nRight; bottom = nBottom;
}

inline CRect::CRect(const RECT& rect)
{
  *(RECT*)this = rect;
}

inline int CRect::Width() const
{
  return right - left + 1;
}

inline int CRect::Height() const
{
  return bottom - top + 1;
}

inline CPoint& CRect::TopLeft() const
{
  return CPoint(left, top);
}

inline CPoint& CRect::BottomRight() const
{
  return CPoint(bottom, right);
}

inline bool CRect::PointInRect(const POINT& point) const
{
  if (point.x >= left && point.x <= right &&
      point.y >= top  && point.y <= bottom)
    return true;
  else
    return false; 
}

inline bool CRect::Overlap(const RECT& rect) const
{
  if ((left - rect.left) > rect.right - rect.left) return false;
  if ((rect.left - left) > Width()) return false;
  if ((top - rect.top) > rect.bottom - rect.top) return false;
  if ((rect.top - top) > Height()) return false;
  return true;
} 

inline void CRect::UnionRect(const RECT& rect1, const RECT& rect2)
{
  assert(rect1.left < rect1.right && rect2.left < rect2.right &&
         rect1.top < rect1.bottom && rect2.top <  rect2.bottom);
  left = Min(rect1.left, rect2.left);
  top  = Min(rect1.top, rect2.top);
  right = Max(rect1.right, rect2.right);
  bottom = Max(rect1.bottom, rect2.bottom);
}

inline void CRect::IntersectRect(const RECT& rect1, const RECT& rect2)
{
  assert(rect1.left < rect1.right && rect2.left < rect2.right &&
         rect1.top < rect1.bottom && rect2.top <  rect2.bottom);
  left = Max(rect1.left, rect2.left);
  top = Max(rect1.top, rect2.top);
  right = Min(rect1.right, rect2.right);
  bottom = Min(rect1.bottom, rect2.bottom);
}

inline bool CRect::operator== (const RECT& rect) const
{
  return (top == rect.top && left == rect.left &&
          bottom == rect.bottom && right == rect.right); 
}

inline bool CRect::operator!= (const RECT& rect) const
{
  return (!operator==(rect));
}

inline void CRect::operator+= (POINT point)
{
  top += point.y; left += point.x;
  bottom += point.y; right += point.x;
}

inline void CRect::operator-= (POINT point)
{
  top -= point.y; left -= point.x;
  bottom -= point.y; right -= point.x;
}

inline CRect CRect::operator+ (POINT point) const
{
  return (CRect(left + point.x, top + point.y,
                right + point.x, bottom + point.y));
}

inline CRect CRect::operator- (POINT point) const
{
  return (CRect(left - point.x, top - point.y,
                right - point.x, bottom - point.y));
}

inline ostream& operator<< (ostream& stream, const CRect& rect) 
{
  cout << "Left/Top: " << rect.left << "/" << rect.top 
       << " Right/Bottom: " << rect.right << "/" << rect.bottom;
  return stream;
}

#endif // _RECT_H
